@@ -27,7 +27,6 @@ class DaskController(AbstractController):
     _controller_name = "dask-local"
     _controller_dask = True
 
-    is_master = True
     client = None
 
     ## Dask controller variables ##
@@ -76,13 +75,6 @@ class DaskController(AbstractController):
         """
         cls.processes = process_count
 
-    @classmethod
-    def sync_processes(self, *args, **kwargs):
-        """
-        This is a thing for KVS. Just return True.
-        """
-        return True
-
     @classmethod
     def check_cluster_state(cls):
         """
