@@ -8,6 +8,12 @@ from inferelator.postprocessing.column_names import (PRECISION_COLUMN, CONFIDENC
 from inferelator.utils import is_string
 import os
 
+import matplotlib
+
+# If matplotlib is being an idiot and trying to set a tkinter backend, switch to agg
+if matplotlib.get_backend() in (i for i in matplotlib.rcsetup.interactive_bk):
+    matplotlib.use('agg')
+
 import matplotlib.pyplot as plt
 
 
