@@ -8,6 +8,12 @@ from inferelator.postprocessing.model_performance import RankSummingMetric
 from inferelator.postprocessing import (TARGET_COLUMN, REGULATOR_COLUMN, PRECISION_COLUMN, RECALL_COLUMN,
                                         CONFIDENCE_COLUMN, GOLD_STANDARD_COLUMN)
 
+import matplotlib
+
+# If matplotlib is being an idiot and trying to set a tkinter backend, switch to agg
+if matplotlib.get_backend() in (i for i in matplotlib.rcsetup.interactive_bk):
+    matplotlib.use('agg')
+
 import matplotlib.pyplot as plt
 
 
